import pytest
import numpy
import io

from gymnasium.spaces import Box
from numpy import array
from unittest import mock


def _(action):
    return numpy.linspace(-1, 1, 2 ** 3)[action]


def test_attributes(three_way_daily_env):
    assert three_way_daily_env.action_space == Box(low=-1, high=1, shape=(2,))
    assert three_way_daily_env.observation_space == Box(low=0., high=float('Inf'), shape=(2, 3))
    assert three_way_daily_env.starting_bank == 10
    assert three_way_daily_env.balance == three_way_daily_env.starting_bank
    assert three_way_daily_env.current_step == 0
    assert numpy.array_equal(three_way_daily_env.bet_size_matrix, numpy.ones(shape=(2, 3)))
    assert numpy.array_equal(three_way_daily_env.teams.values, numpy.array([['FCB', 'PSG'], ['MCB', 'MTA'],
                                                                            ['PSG', 'MCB'], ['FCB', 'MTA'],
                                                                            ['PSG', 'FCB'], ['MTA', 'MCB']]))


@pytest.mark.parametrize("action,expected_reward", [((_(0), _(0)), 0),
                                                    ((_(1), _(1)), -2),
                                                    ((_(3), _(3)), -2),
                                                    ((_(2), _(2)), 0),
                                                    ((_(4), _(4)), 2),
                                                    ((_(5), _(5)), 0),
                                                    ((_(6), _(6)), 2),
                                                    ((_(7), _(7)), 0)])
def test_step(three_way_daily_env, action, expected_reward):
    odds, reward, done, *_ = three_way_daily_env.step(action)
    assert reward == expected_reward
    assert not done
    assert three_way_daily_env.current_step == 1


@pytest.mark.parametrize("current_step,action,expected_reward", [(0, array([_(0), _(0)]), 0),
                                                                 (0, array([_(5), _(6), _(7)]), 0),
                                                                 (0, array([_(2), _(7)]), 2),
                                                                 (1, array([_(4), _(7)]), -1),
                                                                 (1, array([_(4)]), -1),
                                                                 (1, array([_(1), _(3), _(7)]), 2),
                                                                 (2, array([_(2)]), 2),
                                                                 (2, array([_(2), _(2)]), 1),
                                                                 (2, array([_(2), _(4), _(1)]), 3),
                                                                 (2, array([_(3), _(0), _(7)]), 0)])
def test_step_non_uniform(three_way_daily_env_non_uniform, current_step, action, expected_reward):
    three_way_daily_env_non_uniform.current_step = current_step
    odds, reward, done, truncated, info = three_way_daily_env_non_uniform.step(action)
    assert reward == expected_reward


def test_multiple_steps(three_way_daily_env):
    odds, reward, done, truncated, info = three_way_daily_env.step((_(1), _(1)))
    assert reward == -2
    assert three_way_daily_env.balance == three_way_daily_env.starting_bank - 2
    assert not done
    assert three_way_daily_env.current_step == 1
    odds, reward, done, truncated, info = three_way_daily_env.step((_(4), _(4)))
    assert reward == -2
    assert three_way_daily_env.balance == three_way_daily_env.starting_bank - 2 - 2
    assert not done
    assert three_way_daily_env.current_step == 2
    odds, reward, done, truncated, info = three_way_daily_env.step((_(2), _(2)))
    assert reward == -2
    assert three_way_daily_env.balance == three_way_daily_env.starting_bank - 2 - 2 - 2
    assert done


def test_multiple_steps_non_uniform(three_way_daily_env_non_uniform):
    current_bank = three_way_daily_env_non_uniform.starting_bank
    odds, reward, done, truncated, info = three_way_daily_env_non_uniform.step(array([_(2), _(7)]))
    assert three_way_daily_env_non_uniform.balance == current_bank + 2
    assert not done
    assert three_way_daily_env_non_uniform.current_step == 1
    current_bank += 2
    odds, reward, done, truncated, info = three_way_daily_env_non_uniform.step(array([_(1), _(3), _(7)]))
    assert three_way_daily_env_non_uniform.balance == current_bank + 2
    assert not done
    assert three_way_daily_env_non_uniform.current_step == 2
    current_bank += 2
    odds, reward, done, truncated, info = three_way_daily_env_non_uniform.step(array([_(2), _(4), _(1)]))
    assert three_way_daily_env_non_uniform.balance == current_bank + 3
    assert done


def test_render(three_way_daily_env_non_uniform):
    with mock.patch('sys.stdout', new=io.StringIO()) as fake_stdout:
        three_way_daily_env_non_uniform.render()
    assert fake_stdout.getvalue() == 'Home Team FCB VS Away Team PSG, ' \
                                     'Home Team MCB VS Away Team MTA.\nCurrent balance at step 0: 10\n'



import pytest
try:
    from stable_baselines import A2C, ACKTR, DQN, DDPG, PPO1, PPO2, SAC, TD3, TRPO
except ImportError:
    pytest.skip("skipping stable-baselines integration tests because package is not installed",
                allow_module_level=True)


# supported RL algorithms by environment
ENV_DICT = {'BaseOddsEnv': [A2C, ACKTR, DQN, PPO1, PPO2, TRPO],
            'BasePercentageOddsEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO],
            'DailyOddsEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO],
            'DailyPercentageOddsEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO],
            'FootballDataDailyEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO],
            'FootballDataDailyPercentageEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO],
            'TennisDataDailyEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO],
            'TennisDataDailyPercentageEnv': [A2C, ACKTR, DDPG, PPO1, PPO2, SAC, TD3, TRPO]
            }


@pytest.mark.parametrize("alg", ENV_DICT['BaseOddsEnv'])
def test_basic_env_with_(basic_env, alg):
    model = alg('MlpPolicy', basic_env)
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['BasePercentageOddsEnv'])
def test_basic_percentage_env_with_(basic_percentage_env, alg):
    model = alg('MlpPolicy', basic_percentage_env)
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['DailyOddsEnv'])
def test_daily_env_with_(daily_bets_env, alg):
    model = alg('MlpPolicy', daily_bets_env)
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['DailyPercentageOddsEnv'])
def test_daily_percentage_env_with_(daily_bets_percentage_env, alg):
    model = alg('MlpPolicy', daily_bets_percentage_env)
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['FootballDataDailyEnv'])
def test_football_co_uk_daily_env_with_(alg):
    model = alg('MlpPolicy', 'FootballDataDaily-v0')
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['FootballDataDailyPercentageEnv'])
def test_football_co_uk_daily_percentage_env_with_(alg):
    model = alg('MlpPolicy', 'FootballDataDailyPercent-v0')
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['TennisDataDailyEnv'])
def test_tennis_co_uk_daily_env_with_(alg):
    model = alg('MlpPolicy', 'TennisDataDaily-v0')
    model.learn(total_timesteps=10)


@pytest.mark.parametrize("alg", ENV_DICT['TennisDataDailyPercentageEnv'])
def test_tennis_co_uk_daily_percentage_env_with_(alg):
    model = alg('MlpPolicy', 'TennisDataDailyPercent-v0')
    model.learn(total_timesteps=10)



import pytest
try:
    import ray
    from ray.rllib.algorithms.appo import APPOConfig
    # from ray.rllib.agents.es import ESTrainer
    # from ray.rllib.agents.ddpg import TD3Trainer, ApexDDPGTrainer
    # from ray.rllib.agents.ddpg.ddpg import DDPGTrainer
    from ray.rllib.algorithms.impala import ImpalaConfig
    from ray.rllib.algorithms.marwil import MARWILConfig
    # from ray.rllib.agents.pg import PGTrainer
    from ray.rllib.algorithms.ppo import PPOConfig
except ImportError:
    pytest.skip("skipping rllib integration tests because package is not installed",
                allow_module_level=True)


# supported RL algorithms by environment
# ENV_DICT = {'FootballDataDailyEnv': [A2CTrainer, A3CTrainer, ESTrainer, TD3Trainer, ApexDDPGTrainer,
#                                      DDPGTrainer, ImpalaTrainer, MARWILTrainer, PGTrainer, PPOTrainer, APPOTrainer],
#             'FootballDataDailyPercentageEnv': [A2CTrainer, A3CTrainer, ESTrainer, TD3Trainer, ApexDDPGTrainer,
#                                                DDPGTrainer, ImpalaTrainer, MARWILTrainer, PGTrainer, PPOTrainer,
#                                                APPOTrainer],
#             'TennisDataDailyEnv': [A2CTrainer, A3CTrainer, ESTrainer, TD3Trainer, ApexDDPGTrainer,
#                                    DDPGTrainer, ImpalaTrainer, MARWILTrainer, PGTrainer, PPOTrainer, APPOTrainer],
#             'TennisDataDailyPercentageEnv': [A2CTrainer, A3CTrainer, ESTrainer, TD3Trainer, ApexDDPGTrainer,
#                                              DDPGTrainer, ImpalaTrainer, MARWILTrainer, PGTrainer, PPOTrainer,
#                                              APPOTrainer]
#             }

ENV_DICT = {'FootballDataDailyEnv': [APPOConfig, ImpalaConfig, MARWILConfig, PPOConfig],
            'FootballDataDailyPercentageEnv': [APPOConfig, ImpalaConfig, MARWILConfig, PPOConfig],
            'TennisDataDailyEnv': [APPOConfig, ImpalaConfig, MARWILConfig, PPOConfig],
            'TennisDataDailyPercentageEnv': [APPOConfig, ImpalaConfig, MARWILConfig, PPOConfig],
            }

ray.init(logging_level='ERROR', num_cpus=2, num_gpus=0)


def _get_model_config(alg, framework):
    model_config = alg().resources(num_gpus=0, num_learner_workers=1, num_cpus_for_local_worker=1, num_cpus_per_worker=1).debugging(log_level="ERROR").framework(framework=framework)
    return model_config


@pytest.mark.parametrize("alg", ENV_DICT['FootballDataDailyEnv'])
@pytest.mark.parametrize("framework", ['torch'])
def test_football_co_uk_daily_env_with_(alg, framework):
    model_config = _get_model_config(alg, framework)
    model_config = model_config.environment(env="FootballDataDaily-ray-v0")
    model = model_config.build()
    model.train()


@pytest.mark.parametrize("alg", ENV_DICT['FootballDataDailyPercentageEnv'])
@pytest.mark.parametrize("framework", ['torch'])
def test_football_co_uk_daily_percentage_env_with_(alg, framework):
    model_config = _get_model_config(alg, framework)
    model_config = model_config.environment(env="FootballDataDailyPercent-ray-v0")
    model = model_config.build()
    model.train()


@pytest.mark.parametrize("alg", ENV_DICT['TennisDataDailyEnv'])
@pytest.mark.parametrize("framework", ['torch'])
def test_tennis_co_uk_daily_env_with_(alg, framework):
    model_config = _get_model_config(alg, framework)
    model_config = model_config.environment(env="TennisDataDaily-ray-v0")
    model = model_config.build()
    model.train()


@pytest.mark.parametrize("alg", ENV_DICT['TennisDataDailyPercentageEnv'])
@pytest.mark.parametrize("framework", ['torch'])
def test_tennis_co_uk_daily_percentage_env_with_(alg, framework):
    model_config = _get_model_config(alg, framework)
    model_config = model_config.environment(env="TennisDataDailyPercent-ray-v0")
    model = model_config.build()
    model.train()



import shutil
import pytest
import numpy
from gymnasium import envs
from oddsgym.envs.footballdata import FootballDataDailyPercentageEnv
from oddsgym.utils.constants import SupportedSport
from oddsgym.utils.constants.football import CSV_CACHE_PATH
from oddsgym.utils.csv_downloader import create_csv_cache


@pytest.mark.parametrize('kwargs', [{'country': 'England', 'league': 'Premier League', 'start': 1999, 'end': 2011},
                                    {'country': 'France', 'league': 'Premier League', 'start': 2010, 'end': 2011},
                                    {'country': 'England', 'league': 'Premier League', 'start': 2010, 'end': 3000},
                                    {'country': 'England', 'league': 'Premier League', 'start': 2010, 'end': 2009},
                                    {'country': 'Brazil', 'league': 'Premier League', 'start': 2010, 'end': 2011}])
def test_validation(kwargs):
    with pytest.raises(ValueError):
        FootballDataDailyPercentageEnv(config=kwargs)


@pytest.mark.parametrize('optimizer,reward1,reward2,reward3', [('balance', 1, 3.3, 3.3 - 1e-5),
                                                               ('reward', 1, 2.3, -10 * 1e-6)])
def test_env(optimizer, reward1, reward2, reward3):
    env = FootballDataDailyPercentageEnv(config={"optimize": optimizer, "starting_bank": 10})
    assert env is not None
    assert env._extra_odds is not None
    assert env._extra_odds.shape == (380, 46)
    for column in env.ODDS_COLUMNS:
        assert column in env._extra_odds.columns
    action = numpy.zeros(shape=env.action_space.shape)
    action[0] = 0.1
    obs, reward, done, truncated, info = env.step(action)
    assert reward == reward1
    assert obs.shape == (10, 3)
    action = numpy.zeros(shape=env.action_space.shape)
    action[1] = 1 / 11
    obs, reward, done, truncated, info = env.step(action)
    numpy.testing.assert_almost_equal(reward, reward2)
    action = numpy.zeros(shape=env.action_space.shape)
    action[:] = 0.9
    obs, reward, done, truncated, info = env.step(action)
    numpy.testing.assert_almost_equal(reward, reward3)


def test_extra_env():
    env = FootballDataDailyPercentageEnv(config={"extra": True})
    assert env.observation_space.shape == (10, 42)
    action = numpy.zeros(shape=env.action_space.shape)
    action[0] = 0.1
    obs, reward, done, truncated, info = env.step(action)
    assert obs.shape == (10, 42)


def test_registeration():
    spec_ids = [spec.id for spec in envs.registry.values()]
    assert 'FootballDataDaily-v0' in spec_ids
    assert 'FootballDataDailyPercent-v0' in spec_ids


def test_caching():
    # cache directory is empty, so it will use the CSV from the site
    FootballDataDailyPercentageEnv(config={"start": 2019, "end": 2020})
    # create cache directory
    create_csv_cache(True, SupportedSport.football)
    FootballDataDailyPercentageEnv(config={"start": 2019, "end": 2020})
    shutil.rmtree(CSV_CACHE_PATH)



import pytest
import numpy
from gymnasium.spaces import Box


def test_attributes(basic_percentage_env):
    assert basic_percentage_env.action_space == Box(low=numpy.array([-1] * 2),
                                                    high=numpy.array([1.] * 2))
    assert basic_percentage_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 2))
    assert basic_percentage_env.starting_bank == 10
    assert basic_percentage_env.balance == basic_percentage_env.starting_bank
    assert basic_percentage_env.current_step == 0
    assert basic_percentage_env.bet_size_matrix is None


@pytest.mark.parametrize("action,expected_reward", [(numpy.array((0., 0.)), 0),
                                                    (numpy.array((0.25, 0.)), 2.5),
                                                    (numpy.array((0., 0.25)), -2.5),
                                                    (numpy.array((0.25, 0.25)), 0)])
def test_step(basic_percentage_env, action, expected_reward):
    odds, reward, done, *_ = basic_percentage_env.step(action)
    assert reward == expected_reward
    assert done


@pytest.mark.parametrize("action, expected_reward", [(numpy.array((1.1, 0.0)), -11),
                                                     (numpy.array((0.0, 1.1)), -11),
                                                     (numpy.array((0.3, 0.8)), -11)])
def test_legal_bet(basic_percentage_env, action, expected_reward):
    _, reward, _, *_ = basic_percentage_env.step(action)
    assert expected_reward == reward



import shutil
import pytest
import numpy
from gymnasium import envs
from oddsgym.envs.tennisdata import TennisDataDailyPercentageEnv
from oddsgym.utils.constants import SupportedSport
from oddsgym.utils.constants.tennis import CSV_CACHE_PATH
from oddsgym.utils.csv_downloader import create_csv_cache


@pytest.mark.parametrize('kwargs', [{'year': 1999, 'tournament': 'ausopen'},
                                    {'year': 2010, 'tournament': 'masters'},
                                    {'year': 2020, 'tournament': 'ausopen'}])
def test_validation(kwargs):
    with pytest.raises(ValueError):
        TennisDataDailyPercentageEnv(config=kwargs)


@pytest.mark.parametrize('optimizer,reward1,reward2,reward3', [('balance', 0.07, -0.84545454, -0.84546454),
                                                               ('reward', 0.07, -0.91545454, -10 * 1e-6)])
def test_env(optimizer, reward1, reward2, reward3):
    env = TennisDataDailyPercentageEnv(config={"optimize": optimizer, "starting_bank": 10})
    assert env is not None
    assert env._extra_odds is not None
    assert env._extra_odds.shape == (254, 24)
    for column in env.ODDS_COLUMNS:
        assert column in env._extra_odds.columns
    action = numpy.zeros(shape=env.action_space.shape)
    action[0] = 0.1
    obs, reward, done, truncated, info = env.step(action)
    numpy.testing.assert_almost_equal(reward, reward1)
    assert obs.shape == (88, 2)
    action = numpy.zeros(shape=env.action_space.shape)
    action[1] = 1 / 11
    obs, reward, done, truncated, info = env.step(action)
    numpy.testing.assert_almost_equal(reward, reward2)
    action = numpy.zeros(shape=env.action_space.shape)
    action[:] = 0.9
    obs, reward, done, truncated, info = env.step(action)
    numpy.testing.assert_almost_equal(reward, reward3)


def test_extra_env():
    env = TennisDataDailyPercentageEnv(config={"extra": True})
    assert env.observation_space.shape == (88, 20)
    action = numpy.zeros(shape=env.action_space.shape)
    action[0] = 0.1
    obs, reward, done, truncated, info = env.step(action)
    assert obs.shape == (88, 20)


def test_registeration():
    spec_ids = [spec.id for spec in envs.registry.values()]
    assert 'TennisDataDaily-v0' in spec_ids
    assert 'TennisDataDailyPercent-v0' in spec_ids


def test_caching():
    # cache directory is empty, so it will use the CSV from the site
    TennisDataDailyPercentageEnv(config={})
    # create cache directory
    create_csv_cache(False, SupportedSport.tennis)
    TennisDataDailyPercentageEnv(config={})
    shutil.rmtree(CSV_CACHE_PATH)



import os
import shutil
import pytest
import glob
import typer
from typer.testing import CliRunner
from oddsgym.utils.csv_downloader import create_csv_cache
from oddsgym.utils.constants import SPORT_CSV_CACHE_PATHS


app = typer.Typer()
app.command()(create_csv_cache)
runner = CliRunner()


@pytest.mark.parametrize("arguments,expected", (('default', 220),
                                                (['-s', 'football'], 220),
                                                (['-s', 'tennis'], 60),
                                                (['-f', '-s', 'football'], 22),
                                                (['-f', '-s', 'tennis'], 6),
                                                ))
def test_csv_download(arguments, expected):
    sport = arguments[-1] if arguments != 'default' else 'football'
    csv_cache_path = SPORT_CSV_CACHE_PATHS[sport]
    if os.path.exists(csv_cache_path):
        shutil.rmtree(csv_cache_path)
    if arguments == 'default':
        result = runner.invoke(app)
    else:
        result = runner.invoke(app, arguments)
    try:
        assert result.exit_code == 0
        assert len(glob.glob(os.path.join(f'{csv_cache_path}', '**', '*.csv'))) == expected
    except AssertionError:
        raise
    finally:
        shutil.rmtree(csv_cache_path)



import pytest
import numpy
from gymnasium.spaces import Box
from numpy import array


def _(action):
    return numpy.linspace(-1, 1, 2 ** 2)[action]


def test_attributes(tennis_daily_percentage_env):
    assert tennis_daily_percentage_env.action_space == Box(low=array([[-1] * 2] * 3).reshape(6),
                                                           high=array([[1] * 2] * 3).reshape(6))
    assert tennis_daily_percentage_env.observation_space == Box(low=0., high=float('Inf'), shape=(3, 2))
    assert tennis_daily_percentage_env.starting_bank == 10
    assert tennis_daily_percentage_env.balance == tennis_daily_percentage_env.starting_bank
    assert tennis_daily_percentage_env.current_step == 0
    assert numpy.array_equal(tennis_daily_percentage_env.bet_size_matrix, numpy.ones(shape=(3, 2)))
    assert numpy.array_equal(tennis_daily_percentage_env.players.values,
                             numpy.array([['Berrettini M.', 'Harris A.'],
                                          ['Berankis R.', 'Carballes Baena R.'],
                                          ['Cilic M.', 'Moutet C.'],
                                          ['Davidovich Fokina A.', 'Gombos N.'],
                                          ['Hurkacz H.', 'Novak D.'],
                                          ['Querrey S.', 'Berankis R.']]))


@pytest.mark.parametrize("action,expected_reward", [(array([0, 0] * 3).reshape(6), 0),
                                                    (array([0, 0.1] * 3).reshape(6), -2),
                                                    (array([0.1, 0] * 3).reshape(6), 0.84),
                                                    (array([0.1, 0.1] * 3).reshape(6), -1.16)])
def test_step(tennis_daily_percentage_env, action, expected_reward):
    odds, reward, done, *_ = tennis_daily_percentage_env.step(action)
    numpy.testing.assert_almost_equal(reward, expected_reward, 2)
    assert not done
    assert tennis_daily_percentage_env.current_step == 1


def test_multiple_steps(tennis_daily_percentage_env):
    odds, reward, done, truncated, info = tennis_daily_percentage_env.step(array([0, 0.1] * 3).reshape(6))
    assert reward == -2
    assert tennis_daily_percentage_env.balance == tennis_daily_percentage_env.starting_bank - 2
    assert not done
    assert tennis_daily_percentage_env.current_step == 1
    bet_size = 1 / tennis_daily_percentage_env.balance
    odds, reward, done, truncated, info = tennis_daily_percentage_env.step(array([bet_size, 0] * 3).reshape(6))
    numpy.testing.assert_almost_equal(reward, 1.64, 2)
    assert tennis_daily_percentage_env.balance == tennis_daily_percentage_env.starting_bank - 2 + 1.64
    assert not done
    assert tennis_daily_percentage_env.current_step == 2
    bet_size = 1 / tennis_daily_percentage_env.balance
    odds, reward, done, truncated, info = tennis_daily_percentage_env.step(array([0, bet_size] * 3).reshape(6))
    assert reward == -1
    assert tennis_daily_percentage_env.balance == tennis_daily_percentage_env.starting_bank - 2 + 1.64 - 1
    assert done



import pytest
import numpy
import io

from gymnasium.spaces import Box
from unittest import mock


def test_attributes(tennis_env):
    assert tennis_env.action_space.n == 2 ** 2
    assert tennis_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 2))
    assert tennis_env.starting_bank == 10
    assert tennis_env.balance == tennis_env.starting_bank
    assert tennis_env.current_step == 0
    assert numpy.array_equal(tennis_env.bet_size_matrix, numpy.ones(shape=(1, 2)))
    assert numpy.array_equal(tennis_env.players, numpy.array([['Berrettini M.', 'Harris A.'],
                                                              ['Berankis R.', 'Carballes Baena R.']]))


@pytest.mark.parametrize("action,expected_reward", [(0, 0),
                                                    (1, -1),
                                                    (2, 0.11),
                                                    (3, -0.89)])
def test_step(tennis_env, action, expected_reward):
    odds, reward, done, *_ = tennis_env.step(action)
    numpy.testing.assert_almost_equal(reward, expected_reward, 2)
    assert not done
    assert tennis_env.current_step == 1


def test_multiple_steps(tennis_env):
    odds, reward, done, *_ = tennis_env.step(1)
    assert reward == -1
    assert tennis_env.balance == tennis_env.starting_bank - 1
    assert not done
    assert tennis_env.current_step == 1
    odds, reward, done, *_ = tennis_env.step(2)
    numpy.testing.assert_almost_equal(reward, 0.73, 2)
    numpy.testing.assert_almost_equal(tennis_env.balance, tennis_env.starting_bank - 1 + 0.73, 2)
    assert done


def test_render(tennis_env):
    with mock.patch('sys.stdout', new=io.StringIO()) as fake_stdout:
        tennis_env.render()
    assert fake_stdout.getvalue() == 'Player Berrettini M. VS Player Harris A..\nCurrent balance at step 0: 10\n'



import pytest
import numpy
from gymnasium.spaces import Box


def test_attributes(tennis_percentage_env):
    assert tennis_percentage_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 2))
    assert tennis_percentage_env.action_space == Box(low=numpy.array([-1] * 2),
                                                     high=numpy.array([1.] * 2))
    assert numpy.array_equal(tennis_percentage_env.players, numpy.array([['Berrettini M.', 'Harris A.'],
                                                                         ['Berankis R.', 'Carballes Baena R.']]))


@pytest.mark.parametrize("action,expected_reward", [(numpy.array((0, 0)), 0),
                                                    (numpy.array((0.1, 0)), 0.11),
                                                    (numpy.array((0, 0.1)), -1),
                                                    (numpy.array((0.1, 0.1)), -0.89)])
def test_step(tennis_percentage_env, action, expected_reward):
    odds, reward, done, *_ = tennis_percentage_env.step(action)
    numpy.testing.assert_almost_equal(reward, expected_reward, 2)
    assert not done
    assert tennis_percentage_env.current_step == 1



import pytest
import numpy
import io

from gymnasium.spaces import Box
from unittest import mock


def _(action):
    return numpy.linspace(-1, 1, 2 ** 2)[action]


def test_attributes(tennis_daily_env):
    assert tennis_daily_env.action_space == Box(low=-1, high=1, shape=(3,))
    assert tennis_daily_env.observation_space == Box(low=0., high=float('Inf'), shape=(3, 2))
    assert tennis_daily_env.starting_bank == 10
    assert tennis_daily_env.balance == tennis_daily_env.starting_bank
    assert tennis_daily_env.current_step == 0
    assert numpy.array_equal(tennis_daily_env.bet_size_matrix, numpy.ones(shape=(3, 2)))
    assert numpy.array_equal(tennis_daily_env.players.values,
                             numpy.array([['Berrettini M.', 'Harris A.'],
                                          ['Berankis R.', 'Carballes Baena R.'],
                                          ['Cilic M.', 'Moutet C.'],
                                          ['Davidovich Fokina A.', 'Gombos N.'],
                                          ['Hurkacz H.', 'Novak D.'],
                                          ['Querrey S.', 'Berankis R.']]))


@pytest.mark.parametrize("action,expected_reward", [((_(0),), 0),
                                                    ((_(1), _(1)), -2),
                                                    ((_(3), _(3)), -1.16),
                                                    ((_(2), _(2)), 0.84)])
def test_step(tennis_daily_env, action, expected_reward):
    odds, reward, done, *_ = tennis_daily_env.step(action)
    numpy.testing.assert_almost_equal(reward, expected_reward, 2)
    assert not done
    assert tennis_daily_env.current_step == 1


def test_multiple_steps(tennis_daily_env):
    odds, reward, done, truncated, info = tennis_daily_env.step((_(1), _(1)))
    assert reward == -2
    assert tennis_daily_env.balance == tennis_daily_env.starting_bank - 2
    assert not done
    assert tennis_daily_env.current_step == 1
    odds, reward, done, truncated, info = tennis_daily_env.step((_(2), _(2)))
    assert reward == 1.25
    assert tennis_daily_env.balance == tennis_daily_env.starting_bank - 2 + 1.25
    assert not done
    assert tennis_daily_env.current_step == 2
    odds, reward, done, truncated, info = tennis_daily_env.step((_(3), _(3)))
    numpy.testing.assert_almost_equal(-0.66, reward, 2)
    numpy.testing.assert_almost_equal(tennis_daily_env.balance, tennis_daily_env.starting_bank - 2 + 1.25 - 0.66)
    assert done


def test_render(tennis_daily_env):
    with mock.patch('sys.stdout', new=io.StringIO()) as fake_stdout:
        tennis_daily_env.render()
    assert fake_stdout.getvalue() == ('Player Berrettini M. VS Player Harris A., '
                                      'Player Berankis R. VS Player Carballes Baena R..'
                                      '\nCurrent balance at step 0: 10\n')
